package nist.pdpplugin.crypto;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.security.Key;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Enumeration;






import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;
import org.opensaml.xml.security.SecurityHelper;
import org.opensaml.xml.security.x509.X509Credential;




public class AbacKeyManager {

	public AbacKeyManager() {
		// TODO Auto-generated constructor stub
	}
	static Logger log = LogManager.getLogger(AbacKeyManager .class);


	private static final String KEYSTORE_TYPE = "JKS";
	private static final String SECURE_ALGORITHM = "SunX509";


	private KeyManager[] kman;
	private KeyStore keyStore;
	private String keyStorePasswd;
    private String alias;
	private X509Certificate x509Certificate;
	private PublicKey publicKey;
	private PrivateKey privateKey;

	public void generateKeyPair() throws KeyStoreException, UnrecoverableKeyException, NoSuchAlgorithmException{
		Enumeration<String> aliases = this.keyStore.aliases();

        while (aliases.hasMoreElements()) {
        	String alias = (String)aliases.nextElement();
        	this.setAlias(alias);
        	Key key=this.getKeyStore().getKey(alias, this.getKeyStorePasswd().toCharArray());
        	if (key instanceof PrivateKey) {
                // Get certificate of public key
                java.security.cert.Certificate cert = this.getKeyStore().getCertificate(alias);

                // Get public key
                PublicKey publicKey = cert.getPublicKey();
                PrivateKey privateKey=(PrivateKey)key;
                if((publicKey!=null)&&(privateKey!=null)){
                   this.privateKey=privateKey;
                   this.publicKey=publicKey;
                   if(cert instanceof X509Certificate){
                       this.x509Certificate=(X509Certificate) cert;
                  }
                }else{
                	log.fatal("Key Containing Meta cert and private key is not proper. Check System");
                }


        }
	}
	}
	public X509Credential getSigningCredential() throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException{
		
		return SecurityHelper.getSimpleCredential(this.getX509Certificate(), this.getPrivateKey());
	}
	public AbacKeyManager(String keyStoreFilename,
            String keyStorePassword, String keyPassword) throws KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, IOException, UnrecoverableKeyException{
       /* this.privateKey=new ArrayList<PrivateKey>();
        this.publicKey=new ArrayList<PublicKey>();
        this.x509Certificate=new ArrayList<X509Certificate>();*/
		log.debug("MidsKeyManager:init::Initializing key store with file="+keyStoreFilename);
		//System.out.println("Initializing key store with file="+keyStoreFilename);

		this.keyStore = KeyStore.getInstance(KEYSTORE_TYPE);
        this.setKeyStorePasswd(keyStorePassword);

        File file=new File(keyStoreFilename);
        log.debug(keyStoreFilename+"existance status="+file.exists());
		this.keyStore.load(new FileInputStream(file),keyStorePassword.toCharArray() );
		log.debug("MidsKeyManager:init:Loaded file from  key store");
        this.generateKeyPair();
		 Enumeration<String> aliases = this.keyStore.aliases();

         while (aliases.hasMoreElements()) {

             String alias = (String)aliases.nextElement();
             log.debug("MidsKeyManager:init:Inside while loop with alias="+alias);
             Certificate certs =  this.keyStore.getCertificate(alias);
            //System.out.println("Inside while loop with cert size="+certs.);
             if (certs != null) {
                 log.debug("MidsKeyManager:init:Certificate chain '" + alias + "':");
                // for (int c = 0; c < certs.length; c++) {
                     if (certs instanceof X509Certificate) {
                         X509Certificate cert = (X509Certificate)certs;
                         System.out.println(" Certificate "  + ":");
                         System.out.println("  Subject DN: " + cert.getSubjectDN());
                         System.out.println("  Signature Algorithm: " + cert.getSigAlgName());
                         System.out.println("  Valid from: " + cert.getNotBefore() );
                         System.out.println("  Valid until: " + cert.getNotAfter());
                         System.out.println("  Issuer: " + cert.getIssuerDN());
                    // }
                 }
             }
         }

        KeyManagerFactory kmf = KeyManagerFactory.getInstance(SECURE_ALGORITHM);
        if(keyPassword==null)
        	kmf.init( this.keyStore,null);
        else
        kmf.init( this.keyStore,keyPassword.toCharArray());
        kman = kmf.getKeyManagers();
	}
	public AbacKeyManager(URL keyStoreFilename,
            String keyStorePassword, String keyPassword) throws KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, IOException, UnrecoverableKeyException{



		this(keyStoreFilename.getFile(),keyStorePassword,keyPassword);
	}
	/**
	 * @return the kman
	 */
	public KeyManager[] getKman() {
		return kman;
	}
	/**
	 * @param kman the kman to set
	 */
	public void setKman(KeyManager[] kman) {
		this.kman = kman;
	}
	public static void main(String[] args){
		 String keyFile="C:\\Users\\sjha\\keystore.jks";
		 //String keyFile="C:\\dev\\actions\\truststore.jks";

		 try {
			AbacKeyManager deceKeyManager=new AbacKeyManager(keyFile, "12elma","12elma");
			System.out.println("Size of public Key="+deceKeyManager.getPublicKey());
			System.out.println("Size of private Key="+deceKeyManager.getPrivateKey());

		} catch (UnrecoverableKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (KeyStoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CertificateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public KeyStore getKeyStore() {
		return keyStore;
	}
	public void setKeyStore(KeyStore keyStore) {
		this.keyStore = keyStore;
	}
	public String getKeyStorePasswd() {
		return keyStorePasswd;
	}
	public void setKeyStorePasswd(String keyStorePasswd) {
		this.keyStorePasswd = keyStorePasswd;
	}
	public PublicKey getPublicKey() {
		return publicKey;
	}
	public void setPublicKey(PublicKey publicKey) {
		this.publicKey = publicKey;
	}
	public PrivateKey getPrivateKey() {
		return privateKey;
	}
	public void setPrivateKey(PrivateKey privateKey) {
		this.privateKey = privateKey;
	}
	public X509Certificate getX509Certificate() {
		return x509Certificate;
	}
	public void setX509Certificate(X509Certificate x509Certificate) {
		this.x509Certificate = x509Certificate;
	}
	/**
	 * @return the alias
	 */
	public String getAlias() {
		return alias;
	}
	/**
	 * @param alias the alias to set
	 */
	public void setAlias(String alias) {
		this.alias = alias;
	}
}
