package nist.pdpplugin.saml;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import nist.pdpplugin.crypto.AbacKeyManager;

import org.joda.time.DateTime;
import org.opensaml.Configuration;
import org.opensaml.DefaultBootstrap;
import org.opensaml.common.SAMLObjectBuilder;
import org.opensaml.common.SAMLVersion;
import org.opensaml.common.impl.SecureRandomIdentifierGenerator;
import org.opensaml.saml2.core.Attribute;
import org.opensaml.saml2.core.AttributeQuery;
import org.opensaml.saml2.core.Issuer;
import org.opensaml.saml2.core.NameID;
import org.opensaml.saml2.core.Subject;
import org.opensaml.ws.soap.common.SOAPObjectBuilder;
import org.opensaml.ws.soap.soap11.Body;
import org.opensaml.ws.soap.soap11.Envelope;
import org.opensaml.ws.transport.http.HTTPInTransport;
import org.opensaml.ws.transport.http.HTTPOutTransport;
import org.opensaml.ws.transport.http.HttpServletRequestAdapter;
import org.opensaml.ws.transport.http.HttpServletResponseAdapter;
import org.opensaml.xml.ConfigurationException;
import org.opensaml.xml.XMLObjectBuilderFactory;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallerFactory;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.io.UnmarshallerFactory;
import org.opensaml.xml.security.credential.Credential;
import org.opensaml.xml.security.keyinfo.KeyInfoHelper;
import org.opensaml.xml.signature.KeyInfo;
import org.opensaml.xml.signature.Signature;
import org.opensaml.xml.signature.SignatureConstants;
import org.opensaml.xml.signature.SignatureException;
import org.opensaml.xml.signature.Signer;
import org.opensaml.xml.signature.impl.KeyInfoBuilder;
import org.opensaml.xml.util.XMLHelper;
//import org.springframework.mock.web.MockHttpServletRequest;
//import org.springframework.mock.web.MockHttpServletResponse;
import org.w3c.dom.Element;

public class SAML2AttributeQuerBuilder extends SAMLInitializer {
	/*protected XMLObjectBuilderFactory builderFactory;
	*//** Factory for XMLObject marshallers. *//*
    protected MarshallerFactory marshallerFactory;

    *//** Factory for XMLObject unmarshallers. *//*
    protected UnmarshallerFactory unmarshallerFactory;*/
    
    
    
  
	

	
	protected	AbacKeyManager abacKeyManager;
	public SAML2AttributeQuerBuilder(String keyStoreLoc,String storeKey,String aliasKey) throws ConfigurationException, UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, IOException {
	
        super();
        abacKeyManager=new AbacKeyManager(keyStoreLoc, storeKey,aliasKey);
	}
	
	
	 /**
     * Builds a basic attribute query.
     * 
     * @return basic attribute query
	 * @throws NoSuchAlgorithmException 
     */
	 @SuppressWarnings("unchecked")
    protected AttributeQuery buildAttributeQuery(String requester,String subjectUser,String attributeName) throws NoSuchAlgorithmException {
        SAMLObjectBuilder<Issuer> issuerBuilder = (SAMLObjectBuilder<Issuer>) builderFactory
                .getBuilder(Issuer.DEFAULT_ELEMENT_NAME);
        Issuer issuer = issuerBuilder.buildObject();
        issuer.setValue(requester);

        SAMLObjectBuilder<NameID> nameIdBuilder = (SAMLObjectBuilder<NameID>) builderFactory
                .getBuilder(NameID.DEFAULT_ELEMENT_NAME);
        NameID nameId = nameIdBuilder.buildObject();
        nameId.setValue(subjectUser);
        nameId.setFormat(NameID.UNSPECIFIED);

        SAMLObjectBuilder<Subject> subjectBuilder = (SAMLObjectBuilder<Subject>) builderFactory
                .getBuilder(Subject.DEFAULT_ELEMENT_NAME);
        Subject subject = subjectBuilder.buildObject();
        subject.setNameID(nameId);

        
        SAMLObjectBuilder<Attribute> attributeBuilder = (SAMLObjectBuilder<Attribute>) builderFactory
                .getBuilder(Attribute.DEFAULT_ELEMENT_NAME);
        Attribute attribute = attributeBuilder.buildObject();
        attribute.setName(attributeName);
        SAMLObjectBuilder<AttributeQuery> attributeQueryBuilder = (SAMLObjectBuilder<AttributeQuery>) builderFactory
                .getBuilder(AttributeQuery.DEFAULT_ELEMENT_NAME);
        AttributeQuery query = attributeQueryBuilder.buildObject();
        
        query.getAttributes().add(attribute);
        SecureRandomIdentifierGenerator srifg= new SecureRandomIdentifierGenerator ();
        query.setID(srifg.generateIdentifier());
        query.setIssueInstant(new DateTime());
        query.setIssuer(issuer);
        query.setSubject(subject);
        query.setVersion(SAMLVersion.VERSION_20);

        return query;
    }
    /**
    * Wraps an attribute query in a SOAP message, marshalls, and serializes it.
    * 
    * @param query the attribute query to wrap
    * 
    * @return the SOAP message
    * 
    * @throws MarshallingException thrown if the message can not be marshalled
    */
   @SuppressWarnings("unchecked")
   public String getSOAPMessage(AttributeQuery query) throws MarshallingException {
       SOAPObjectBuilder<Body> bodyBuilder = (SOAPObjectBuilder<Body>) builderFactory
               .getBuilder(Body.DEFAULT_ELEMENT_NAME);
       Body body = bodyBuilder.buildObject();
       body.getUnknownXMLObjects().add(query);

       SOAPObjectBuilder<Envelope> envelopeBuilder = (SOAPObjectBuilder<Envelope>) builderFactory
               .getBuilder(Envelope.DEFAULT_ELEMENT_NAME);
       Envelope envelope = envelopeBuilder.buildObject();
       envelope.setBody(body);

       Marshaller marshaller = marshallerFactory.getMarshaller(envelope);
       Element envelopeElem = marshaller.marshall(envelope);

       StringWriter writer = new StringWriter();
       XMLHelper.writeNode(envelopeElem, writer);
       return writer.toString();
   }
   private Signature createSignature() throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException{
	   Signature signature = (Signature) Configuration.getBuilderFactory()
			   .getBuilder(Signature.DEFAULT_ELEMENT_NAME)
			   .buildObject(Signature.DEFAULT_ELEMENT_NAME);
       Credential signingCredential=this.abacKeyManager.getSigningCredential();
	   signature.setSigningCredential( signingCredential);
	   signature.setSignatureAlgorithm(SignatureConstants.ALGO_ID_SIGNATURE_RSA_SHA1);
	   signature.setCanonicalizationAlgorithm(SignatureConstants.ALGO_ID_C14N_EXCL_OMIT_COMMENTS);
	   signature.setKeyInfo(getKeyInfo(signingCredential));

	   //assertion.setSignature(signature);
	   return signature;
   }
   private KeyInfo getKeyInfo(Credential signingCredential)  {
	   
	   KeyInfoBuilder kif=(KeyInfoBuilder) builderFactory
               .getBuilder(KeyInfo.DEFAULT_ELEMENT_NAME);
	   
	   KeyInfo ki=kif.buildObject();
	  // KeyInfo ki=(KeyInfo)createOpenSAMLObject(KeyInfo.DEFAULT_ELEMENT_NAME);
	  /* BasicX509Credential credential=new BasicX509Credential();
	   credential.setEntityCertificate(certificate);*/
	   KeyInfoHelper.addPublicKey(ki,signingCredential.getPublicKey());
	   return ki;
	 }
   private AttributeQuery signAtttributeQuery() throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException{
	   
	   Signature signature=this.createSignature();
	   AttributeQuery attributeQuery=buildAttributeQuery("plugin","Dwayne","Citizenship");
	   attributeQuery.setSignature(signature);
	   return attributeQuery;
   }
public AttributeQuery signAtttributeQuery(String requester,String subjectUser,String attributeName) throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException, MarshallingException, SignatureException{
	   
	   Signature signature=this.createSignature();
	   AttributeQuery attributeQuery=buildAttributeQuery(requester,subjectUser,attributeName);
	   attributeQuery.setSignature(signature);
	   Configuration.getMarshallerFactory().getMarshaller(attributeQuery).marshall(attributeQuery);
	   Signer.signObject(signature);
	   return attributeQuery;
   }
   public String provideSignedAttributeInSoapMessage() throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException, MarshallingException{
	   AttributeQuery attrQuery=this.signAtttributeQuery();
	  return this.getSOAPMessage(attrQuery);
	   
   }
}
