package nist.pdpplugin.saml.metadata;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.opensaml.saml2.metadata.AttributeAuthorityDescriptor;
import org.opensaml.saml2.metadata.EntityDescriptor;
import org.opensaml.saml2.metadata.IDPSSODescriptor;
import org.opensaml.saml2.metadata.SingleLogoutService;
import org.opensaml.saml2.metadata.provider.FilesystemMetadataProvider;
import org.opensaml.saml2.metadata.provider.MetadataProviderException;
import org.opensaml.xml.ConfigurationException;
import org.opensaml.xml.parse.BasicParserPool;
import org.opensaml.xml.parse.ParserPool;

public class IDPMetadataReader {

	ByteArrayInputStream metadata=null;
	private ParserPool parser;
	static Logger log = LogManager.getLogger(IDPMetadataReader.class);
	FileInputStream fis = null;
	String metaDataLocation=null;
	FilesystemMetadataProvider filesystemMetadataProvider=null;
	
	public IDPMetadataReader() {
		// TODO Auto-generated constructor stub
	}
	public String getMetaDataLocation() {
		return metaDataLocation;
	}


	public void setMetaDataLocation(String metaDataLocation) {
		this.metaDataLocation = metaDataLocation;
	}


	public IDPMetadataReader(String metaDataLoc) throws IOException, MetadataProviderException{
		// A Db call to get meta data. Currently trying to read using FIle

        this.setMetaDataLocation(metaDataLoc);
        filesystemMetadataProvider=new FilesystemMetadataProvider(new File(this.getMetaDataLocation()));
        this.filesystemMetadataProvider.setParserPool(new BasicParserPool());
        this.filesystemMetadataProvider.initialize();

	}
   
   public SingleLogoutService getSingleLogoutService(String identityProvider,String assertionURL) throws MetadataProviderException{
	   EntityDescriptor idpDescriptor=this.filesystemMetadataProvider.getEntityDescriptor(identityProvider);
	   if(idpDescriptor==null){
		   return null;
	   }
	   IDPSSODescriptor idpSSODescriptor = idpDescriptor.getIDPSSODescriptor
		("urn:oasis:names:tc:SAML:2.0:protocol");
	   List<SingleLogoutService> listSLOService=idpSSODescriptor.getSingleLogoutServices();
	   
	   for(SingleLogoutService singleLogutService:listSLOService){
		  // System.out.println("Acs provided="+)
		   if(singleLogutService.getLocation().equalsIgnoreCase(assertionURL))
		   return singleLogutService;
	   }
	   return null;
	   
   }
   public AttributeAuthorityDescriptor getAttributeAuthorityDescriptor(String identityProvider) throws MetadataProviderException{
	   EntityDescriptor idpDescriptor=this.filesystemMetadataProvider.getEntityDescriptor(identityProvider);
	   if(idpDescriptor==null){
		   return null;
	   }
	   /*IDPSSODescriptor idpSSODescriptor = idpDescriptor.getIDPSSODescriptor
		("urn:oasis:names:tc:SAML:2.0:protocol");*/
	   AttributeAuthorityDescriptor attributeAuthorityDescriptor  =idpDescriptor.getAttributeAuthorityDescriptor(org.opensaml.common.xml.SAMLConstants.SAML20P_NS);
	   System.out.println("Attribute Descriptor status"+attributeAuthorityDescriptor==null);
	   /*for(SingleLogoutService singleLogutService:listSLOService){
		  // System.out.println("Acs provided="+)
		   if(singleLogutService.getLocation().equalsIgnoreCase(assertionURL))
		   return singleLogutService;
	   }*/
	   return attributeAuthorityDescriptor;
	   
   }
   public String getAttributeAuthorityDescriptorLocation(String identityProvider) throws MetadataProviderException{
	   
	   AttributeAuthorityDescriptor attributeAuthorityDescriptor=this.getAttributeAuthorityDescriptor(identityProvider);
	   return attributeAuthorityDescriptor.getAttributeServices().get(0).getLocation();
   }
   public List<SingleLogoutService> getSingleLogoutService(String identityProvider) throws MetadataProviderException{
	   EntityDescriptor idpDescriptor=this.filesystemMetadataProvider.getEntityDescriptor(identityProvider);
	   if(idpDescriptor==null){
		   return null;
	   }
	   IDPSSODescriptor idpSSODescriptor = idpDescriptor.getIDPSSODescriptor
		("urn:oasis:names:tc:SAML:2.0:protocol");
	   List<SingleLogoutService> listSLOService=idpSSODescriptor.getSingleLogoutServices();
	   return listSLOService;
	   
	   
	   //return null;
	   
   }

   public static void main(String[] args){
	   try {
		   org.opensaml.DefaultBootstrap.bootstrap();
		   //IDPMetadataReader nccoeMetaDataProvider=new IDPMetadataReader("C:\\Users\\sjha\\Documents\\porjects\\nist\\idpMetadata1.xml");
		   IDPMetadataReader nccoeMetaDataProvider=new IDPMetadataReader("C:\\Users\\sjha\\Documents\\artifact\\AAD-metadata-IDP.xml");
		   EntityDescriptor entityDescriptor=nccoeMetaDataProvider.getFilesystemMetadataProvider().getEntityDescriptor("https://rp.abac.test:9031");
		    if(entityDescriptor!=null){
              System.out.println("Entity Descriptor was not null");
		   }
		    String s=  nccoeMetaDataProvider.getAttributeAuthorityDescriptorLocation("https://rp.abac.test:9031");
		    System.out.println("Atrribute service="+s);
		//XMLObject xmlObject=deceMetaDataProvider.getMetadata();
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (MetadataProviderException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (ConfigurationException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
   }


public FilesystemMetadataProvider getFilesystemMetadataProvider() {
	return filesystemMetadataProvider;
}


public void setFilesystemMetadataProvider(
		FilesystemMetadataProvider filesystemMetadataProvider) {
	this.filesystemMetadataProvider = filesystemMetadataProvider;
}

}
