package nist.pdpplugin.transport;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.HttpClientConnectionManager;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.util.EntityUtils;

public class HTTPSTransmitter {

	CloseableHttpClient httpClient;
	 URL  urlTLS;
	 URL  urlTrust;
	 Properties prop;
	 String tlsPassword;
	 String trustPass;
	public void init() throws IOException{
		
		ClassLoader classLoader = HTTPSTransmitter.class.getClassLoader();
		urlTLS= classLoader.getResource("abacClient.jks");
		  
		urlTrust=classLoader.getResource("truststore.jks");
		
		   
		 InputStream input = new FileInputStream(classLoader.getResource("pdplugin.properties").getPath());
		 prop=new Properties();  
			// load a properties file
			prop.load(input);
			
        
        tlsPassword=prop.getProperty("TlskeyPassowrd");
		trustPass=prop.getProperty("TrustStorePassowrd");
	}
	public HTTPSTransmitter() throws KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, IOException, UnrecoverableKeyException, KeyManagementException {
		//String keyStoreLoc,String keyStorePass,String trustStoreLoc,String trustStorePass
		  init();
		  KeyManagerFactory kmf;
		  TrustManagerFactory tmf;
		// read in the keystore from the filesystem, this should contain a single keypair
				KeyStore clientKeyStore = KeyStore.getInstance(KeyStore.getDefaultType());
				clientKeyStore.load(new FileInputStream(urlTLS.getPath()),

				        tlsPassword.toCharArray());
		 
				KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
				trustStore.load(new FileInputStream(		urlTrust.getPath()),
						trustPass.toCharArray());
				
				
				
				 // Trust own CA and all self-signed certs
				//SSLContext sslcontext = SSLContext.getInstance(arg0)
				
		      /*  SSLContext sslcontext = SSLContexts.custom()
		                .loadTrustMaterial(trustStore, new TrustSelfSignedStrategy())
		                .build();*/
				SSLContext sslContext=SSLContexts.createDefault();
				
				 kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
				 kmf.init(clientKeyStore, tlsPassword.toCharArray());
				
				 tmf=TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
				 tmf.init(trustStore);
				 sslContext.init(kmf.getKeyManagers(), tmf.getTrustManagers(), new SecureRandom());
				
				// Allow TLSv1 protocol only
				 SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext,NoopHostnameVerifier.INSTANCE);
				 //sslsf.s
			      /*  SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(
			                sslContext,
			                new String[] { "TLSv1" },
			                null,
			                SSLConnectionSocketFactory.BROWSER_COMPATIBLE_HOSTNAME_VERIFIER);*/
			        CloseableHttpClient httpclient = HttpClients.custom()
			                .setSSLSocketFactory(sslsf)
			                .build();
			      
		 
				// create and configure scheme registry
				//SchemeRegistry registry = new SchemeRegistry();
			      
			        Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory>create()
			                			                .register("https", sslsf)
			                .build();
		 
				// create a client connection manager to use in creating httpclients
			        HttpClientConnectionManager cm = new PoolingHttpClientConnectionManager( registry );
			     
		 
				// create the client based on the manager, and use it to make the call
			        this.httpClient= HttpClients.custom()
			                .setConnectionManager(cm)
			                .build();
	}

	public void doPost(String url ) throws ClientProtocolException, IOException {

	  
	    
		HttpPost httpPost = new HttpPost(url);
		List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		nvps.add(new BasicNameValuePair("Requester", "urn:nccoe:abac:plugin"));
		nvps.add(new BasicNameValuePair("SubjectId", "jwilliams"));
		nvps.add(new BasicNameValuePair("AttributeName", "email"));
		httpPost.setEntity(new UrlEncodedFormEntity(nvps));
		CloseableHttpResponse response2 = this.httpClient.execute(httpPost);

		try {
		    System.out.println(response2.getStatusLine());
		    HttpEntity entity2 = response2.getEntity();
		    // do something useful with the response body
		    // and ensure it is fully consumed
		    EntityUtils.consume(entity2);
		} finally {
		    response2.close();
		}
	}
	public static void main(String[] args) throws UnrecoverableKeyException, KeyManagementException, KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, IOException {
		String url="https://localhost:8443/SAMLProxy-0.0.1-SNAPSHOT/samlproxy/";
		//String url="https://localhost:8443/samlproxy/";
	    HTTPSTransmitter  hTTPSTransmitter= new HTTPSTransmitter();
	    hTTPSTransmitter.doPost(url);
	    
	}

}
