package nist.pdpplugin.transport;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.net.URL;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import nist.pdpplugin.crypto.AbacX509TrustManager;
import nist.pdpplugin.crypto.TLSKeyManager;
import nist.pdpplugin.saml.SAML2AttributeQuerBuilder;
import nist.pdpplugin.saml.SoapSAML2;
import nist.pdpplugin.saml.metadata.IDPMetadataReader;

import org.opensaml.saml2.core.Response;
import org.opensaml.saml2.metadata.EntityDescriptor;
import org.opensaml.saml2.metadata.provider.MetadataProviderException;
import org.opensaml.ws.soap.client.BasicSOAPMessageContext;
import org.opensaml.ws.soap.client.http.HttpClientBuilder;
import org.opensaml.ws.soap.client.http.HttpSOAPClient;
import org.opensaml.ws.soap.client.http.TLSProtocolSocketFactory;
import org.opensaml.ws.soap.common.SOAPException;
import org.opensaml.ws.soap.soap11.Envelope;
import org.opensaml.xml.ConfigurationException;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.signature.SignatureException;
import org.opensaml.xml.util.Base64;
import org.opensaml.xml.util.XMLHelper;
import org.w3c.dom.Element;

public class SoapHTTPTransmitter {
	BasicSOAPMessageContext soapContext=null;
	ClassLoader classLoader = null;
	URL urlTLS;
	URL urlSignature;
	URL urlTrust;
	URL urlMetadata;
	String  idpIssuer;
	String	spIssuer;
	String	signingKeyPassword;
	String	TlskeyPassowrd;
	String	TrustStorePassowrd;
	
	
	/**
	 * @return the idpIssuer
	 */
	public String getIdpIssuer() {
		return idpIssuer;
	}

	/**
	 * @param idpIssuer the idpIssuer to set
	 */
	public void setIdpIssuer(String idpIssuer) {
		this.idpIssuer = idpIssuer;
	}

	/**
	 * @return the spIssuer
	 */
	public String getSpIssuer() {
		return spIssuer;
	}

	/**
	 * @param spIssuer the spIssuer to set
	 */
	public void setSpIssuer(String spIssuer) {
		this.spIssuer = spIssuer;
	}

	/**
	 * @return the signingKeyPassword
	 */
	public String getSigningKeyPassword() {
		return signingKeyPassword;
	}

	/**
	 * @param signingKeyPassword the signingKeyPassword to set
	 */
	public void setSigningKeyPassword(String signingKeyPassword) {
		this.signingKeyPassword = signingKeyPassword;
	}

	/**
	 * @return the tlskeyPassowrd
	 */
	public String getTlskeyPassowrd() {
		return TlskeyPassowrd;
	}

	/**
	 * @param tlskeyPassowrd the tlskeyPassowrd to set
	 */
	public void setTlskeyPassowrd(String tlskeyPassowrd) {
		TlskeyPassowrd = tlskeyPassowrd;
	}

	/**
	 * @return the trustStorePassowrd
	 */
	public String getTrustStorePassowrd() {
		return TrustStorePassowrd;
	}

	/**
	 * @param trustStorePassowrd the trustStorePassowrd to set
	 */
	public void setTrustStorePassowrd(String trustStorePassowrd) {
		TrustStorePassowrd = trustStorePassowrd;
	}
	
	
	
	
	Properties prop;
	InputStream input;

	public SoapHTTPTransmitter() throws IOException {
		   prop = new Properties();
		   soapContext = new BasicSOAPMessageContext();
		   classLoader = this.getClass().getClassLoader();
		   urlTLS= classLoader.getResource("abacClient.jks");
		   urlSignature= classLoader.getResource("abacSigningClient.jks");
		   urlTrust=classLoader.getResource("truststore.jks");
		   urlMetadata=classLoader.getResource("metadata.xml");
		   
		   input = new FileInputStream(classLoader.getResource("pdplugin.properties").getPath());
		   
			// load a properties file
			prop.load(input);
			
           this.setSigningKeyPassword(prop.getProperty("signingKeyPassword"));
           this.setTlskeyPassowrd(prop.getProperty("TlskeyPassowrd"));
		   this.setTrustStorePassowrd(prop.getProperty("TrustStorePassowrd"));
		   
		  /*Envelope envelope = buildSOAP11Envelope(request);
		  soapContext.setOutboundMessage(envelope);*/
	}
	
	public Map<String, List<String>> transmit(String requester,String subjectUser,String attributeName) throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, ConfigurationException, IOException, MetadataProviderException, org.opensaml.xml.security.SecurityException, MarshallingException, SignatureException{
		//Change following lines to read from property file
		String keyFile=urlSignature.getPath();//"C:\\Users\\sjha\\keystore.jks";
		System.out.println("KeyFile="+keyFile);
		
		
		//String tlsKeyFile="C:\\Users\\sjha\\tlsKeyStore.jks";
		String tlsKeyFile=urlTLS.getPath();
		System.out.println("TLS Key File Path="+tlsKeyFile);
		
		
		String trustKeyFile=urlTrust.getPath();
		System.out.println("Trust Key File Path="+tlsKeyFile);
	
  	    //Build SAML Attribute Query, 
		
		SAML2AttributeQuerBuilder saml2AttributeQuerBuilder=new SAML2AttributeQuerBuilder(keyFile, this.getSigningKeyPassword(),this.getSigningKeyPassword());
		
  	    XMLObject xmlObject=saml2AttributeQuerBuilder.signAtttributeQuery(requester, subjectUser, attributeName);
  	    //XMLHelper.prettyPrintXML(xmlObject.getDOM());
  	    
  	    System.out.println("Attribute Query Response= "+this.createSuccessResponse(xmlObject));
  	    SoapSAML2 soapSAML2=new SoapSAML2();
  	    Envelope outBoundEnvelope= soapSAML2.buildSOAP11Envelope(xmlObject);
  	    System.out.println( outBoundEnvelope.toString());
  	   // soapSAML2.parseEnvelope(outBoundEnvelope);
  	    soapContext.setOutboundMessage(outBoundEnvelope);
  	  
  	    //Read End Point of of IDP Attribute Descriptor
  	    //IDPMetadataReader nccoeMetaDataProvider=new IDPMetadataReader("C:\\Users\\sjha\\Documents\\porjects\\nist\\idpMetadata1.xml");
  	    IDPMetadataReader nccoeMetaDataProvider=new IDPMetadataReader(urlMetadata.getPath());
  	    System.out.println("Path="+urlMetadata.getPath());
	    EntityDescriptor entityDescriptor=nccoeMetaDataProvider.getFilesystemMetadataProvider().getEntityDescriptor("https://10.33.7.8:9031");
	    if(entityDescriptor!=null){
            System.out.println("Entity Descriptor was not null");
	     }else{
	    	 System.out.println("Entity Descriptor was  null");
	     }
	    String serverEndpoint=  nccoeMetaDataProvider.getAttributeAuthorityDescriptorLocation("https://10.33.7.8:9031");
  	    System.out.println("End Point="+serverEndpoint);
	    //Build Mutual TLS authentcation.
  	   
  	    
  	    TLSKeyManager abacKeyManager=new TLSKeyManager(tlsKeyFile,this.getTlskeyPassowrd(),this.getTlskeyPassowrd());
  	    HttpClientBuilder clientBuilder = new HttpClientBuilder();
  	    System.out.println("TrustStorePassword="+this.getTrustStorePassowrd());
        clientBuilder.setHttpsProtocolSocketFactory(
              new TLSProtocolSocketFactory(abacKeyManager, new AbacX509TrustManager(trustKeyFile,this.getTrustStorePassowrd(),this.getTrustStorePassowrd())));
       
        HttpSOAPClient soapClient = new HttpSOAPClient(clientBuilder.buildClient(), soapSAML2.getParserPool());
       
      // Send the message
      try {
    	  System.out.println("Is Client null="+(soapClient==null)+" serverEnpoint="+serverEndpoint+" soapContext is null="+(soapContext==null));
          soapClient.send(serverEndpoint, soapContext);
      } catch (SOAPException e) {
          e.printStackTrace();
      } catch (SecurityException e) {
          e.printStackTrace();
      }
       
      // Access the SOAP response envelope
      Envelope soapResponse = (Envelope) soapContext.getInboundMessage();
       
      System.out.println("SOAP Response was:");
      System.out.println(XMLHelper.prettyPrintXML(soapResponse.getDOM()));
      
      return soapSAML2.getAttributes(soapResponse);
  	    
  	  
	}
	 
		public String createSuccessResponse(XMLObject xmlObject)
		throws MarshallingException {
			// Now we must build our representation to put into the html form to be
			// submitted to the idp
			Marshaller marshaller = org.opensaml.xml.Configuration
			.getMarshallerFactory().getMarshaller(xmlObject);
			Element authDOM = marshaller.marshall(xmlObject);
			StringWriter rspWrt = new StringWriter();
			XMLHelper.writeNode(authDOM, rspWrt);
			String responseXML = rspWrt.toString();
			//log.info("responseXML=" + responseXML);
			/*String samlResponse = new String(Base64.encodeBytes(responseXML
					.getBytes(), Base64.DONT_BREAK_LINES));*/

			// Set the URL to where we resolved the destination to go to.
			//this.setActionURL(xmlObject.getDestination());

			return responseXML;
		}
	 public static void main( String[] args ) throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, ConfigurationException, IOException, MetadataProviderException, org.opensaml.xml.security.SecurityException, MarshallingException, SignatureException{
		 SoapHTTPTransmitter soapHTTPTransmitter=new SoapHTTPTransmitter();
		 soapHTTPTransmitter.transmit("urn:nccoe:abac:plugin","jwilliams","email");
	 }
}
